/*Version 3.1
 *========================
 *ATENCION
 *========================
 *
 * Debido a que utilizamos mutex (#include<mutex>) es necesario compilar con la opción -std=c++0x
 * Hay que utilizar el espacio de nombres en este caso std::mutex, si solo utilizamos mutex
 * el compilador se queja diciendo que no sabe lo que es mutex
 *
 * Para alguna versión de compilador mingw, se debe de definir el simbolo/identificador del procesador (-D)
 * _GLIBCXX_PROFILE (-D_GLIBCXX_PROFILE) para la std::map
 *========================
 *FIN ATENCION
 *========================
 *
 * Ejempo:
 *
 * CContadorMemoria *lpContMemoria;
 * std::string *lpstrError;
 *
 * como es la primera vez crea la instancia de CContadorMemoria y la devuelve
 * lpContMemoria=CContadorMemoria::getInstance();
 *
 * variable=new tipo;
 * lpContMemoria->crear((UINT_PTR)variable,__FILE__,__func__,__LINE__-1,"TEXTO_LIBRE");
 *
 * delete variable;
 * lpContMemoria->liberar((UINT_PTR)variable);
 *
 * Devolvera un stdString en caso de no haber memoria sin liberar.
 * Si se ha liberado correctamente la memoria devolvera NULL
 * lpstrError=CContadorMemoria::finalizar();
 * if (lpstrError!=NULL)
 * {
 *  muestro el lpstrError
 * }
 *
 *
 *
 */


#ifndef CMEMORYLEAKS_H_
#define CMEMORYLEAKS_H_

#include <mutex>
#include <map>
#include <string>
//para UINT_PTR
#include <basetsd.h>



class CMemoryLeaks
{
 private:
  //constructor y destructor lo definimos en la parte privada,
  //para que no puedan ser utilizados
  CMemoryLeaks(void);
  ~CMemoryLeaks(void);
  //prohibimos el uso de los operadores constructor de copia y asignación
  CMemoryLeaks(CMemoryLeaks const&)=delete;
  CMemoryLeaks& operator=(CMemoryLeaks const&)=delete;

  static CMemoryLeaks *mspInstancia;
  static std::mutex msMutex;
  //mapa donde dejaremos los registros a traves de CContadorMemoria::crear
  std::map <UINT_PTR,std::string> mpMap;


  std::map <UINT_PTR,std::string> getMap(void);


 public:
  static CMemoryLeaks *getInstance(void);

  //Funcion que se encargará de hacer el borrado de la instancia
  static std::string *finalizar(void);


  //añadimos un registro en mMap cuando creemos un espacio de memoria (new)
  void crear(UINT_PTR pDirMem,std::string pstrFile,
	   std::string pstrfuncion, unsigned int puiLine,
	   std::string pstrInf);

  //borrar el registro insertado en mMap cuando liberemos memoria (delete)
  void liberar(UINT_PTR pDirMem);
};
#endif
